﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using MyECULib;

namespace MyECUPlug
{
    public partial class MyECUplug :  Form, MyECUDataObserver
    {
        int chRPM = -1;
        int chTPS = -1;
        int chInjDur = -1;
        int chSpkAdv = -1;
        int chSoI = -1;
        int chEoI = -1;
        int chAirP = -1;
        int chOilT = -1;
        int chAirT = -1;
        int chBatV = -1;
        int chO2v = -1;
        int chAFR = -1;
        int chLambda = -1;
        //int chTargV = -1;
        int chTargAFR0 = -1;
        int chTargLambda0 = -1;
        int chO2Adjust0 = -1;
        int chMapX = -1;
        int chMapY = -1;
        int chDutyCycle = -1;
        int chBaroAdj = -1;

        bool lwChannelsCreated = false;
        MyECUSource com;
        MyECUDataDispatcher dispatcher;
        private bool dispatcherStopping = false;

        uint DeadTime = 0;

        uint numalerts = 0;
        int ThrottleValue = 0;
        string VersionString = "";

        public MyECUplug()
        {
            InitializeComponent();
            UpdatePortList();
            lwConnect();
            timerUpdatePortList.Enabled = true;
            progressBarTHR.Minimum = 0;
            progressBarTHR.Maximum = 1023;
        }

        private void MyECUForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (dispatcher != null)
            {
                dispatcher.stop();
                dispatcher = null;
            }
        }

        public void MyECUSourceClosed()
        {
            com.Close();
            dispatcher = null;
            dispatcherStopping = true;
        }

        private void UpdatePortList()
        {
            string[] names = MyECUSerial.getPortNames();
            string selected = (String)listBoxInputPort.SelectedItem;
            listBoxInputPort.Items.Clear();
            foreach (string name in names)
            {
                listBoxInputPort.Items.Add(name);
            }
            if (selected != null)
            {
                listBoxInputPort.SelectedItem = selected;
            }
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            if (com == null) 
            {
                string inputSerialPortName = (String)listBoxInputPort.SelectedItem;
                if ((inputSerialPortName == null) || (inputSerialPortName.Length == 0))
                {
                    MessageBox.Show("Please decide on a serial port first");
                    return;
                }
                                
                com = new MyECUSerial(inputSerialPortName);

                saveFileDialog1.FileName = TimeStampFilename();
                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    if (!com.setOutputFile(saveFileDialog1.FileName))
                    {
                        MessageBox.Show("Could not open output file");
                    }
                }

                if (com.Open())
                {
                    timerUpdatePortList.Enabled = false;
                    buttonConnect.Text = "Disconnect";
                    dispatcher = new MyECUDataDispatcher(com);
                    dispatcher.addObserver(this);
                    com.DiscardBuffer();
                    dispatcher.startup();
                    timerUpdateUI.Enabled = true;
                }
                else
                {
                    teardown();
                    MessageBox.Show("Could not connect to serial port " + inputSerialPortName);
                }
            }
            else
            {
                if (dispatcher != null)
                {
                    dispatcher.stop();
                    dispatcher = null;
                }
                else
                {
                    MessageBox.Show("Weird error: com is not null but dispatcher is!!");
                }
                teardown();
            }
        }

        private void teardown()
        {
            com.Close();
            com = null;
            if (dispatcher != null)
            {
                dispatcher.stop();
                dispatcher = null;
            }
            timerUpdateUI.Enabled = false;
            UpdatePortList();
            buttonConnect.Text = "Connect";
            timerUpdatePortList.Enabled = true;
            progressBarTHR.Value = 0;
            buttonConnect.Enabled = true;
            DeadTime = 0;
            ThrottleValue = 0;
            numalerts = 0;
            VersionString = "";
            SendToChannel(chRPM, 0);
            SendToChannel(chTargAFR0, 0);
            SendToChannel(chTargLambda0, 0);
            SendToChannel(chBaroAdj, 0);
            //SendToChannel(chTargV, 0);
            SendToChannel(chO2Adjust0, 0);
            this.Text = "MyECU plugin for LogWorks";
        }

        private string TimeStampFilename()
        {
            return DateTime.Now.ToString("yyMMddHHmm") + ".dmp";
        }

        // Progressbar is hidden button for open file
        private void progressBar1_Click_1(object sender, EventArgs e)
        {
            // refuse to open a file while we're listening to serial port
            if (timerUpdateUI.Enabled)
            {
                return;
            }

            // if we're already reading a file, stop that
            if (com != null)
            {
                dispatcher = null;
                dispatcherStopping = true;
                return;
            }

            DialogResult result = openFileDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                String fn = openFileDialog1.FileName;
                com = new MyECUFile(fn,30);

                saveFileDialog1.FileName = TimeStampFilename();
                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    if (!com.setOutputFile(saveFileDialog1.FileName))
                    {
                        MessageBox.Show("Could not open output file");
                    }
                }

                if (com.Open())
                {
                    buttonConnect.Enabled = false;
                    dispatcher = new MyECUDataDispatcher(com);
                    dispatcher.addObserver(this);
                    com.DiscardBuffer();
                    dispatcher.startup();
                    timerUpdateUI.Enabled = true;
                }

                else
                {
                    teardown();
                    MessageBox.Show("Failed to open file!");
                }
            }
        }

        public void MyECUSourceUpdate(MyECUData data)
        {
            switch (data.type)
            {
                case MyECUData.types.TimingInfo:
                    SendToChannel(chRPM, data.ti.RPM());
                    SendToChannel(chInjDur, data.ti.InjDuration0);   // This is the final InjDuration (except for deadtime)
                    SendToChannel(chSpkAdv, data.ti.SkpAdv());
                    SendToChannel(chSoI, data.ti.InjStart(DeadTime));
                    SendToChannel(chEoI, data.ti.InjEnd());
                    SendToChannel(chDutyCycle, data.ti.DutyCycle(DeadTime));
                    SendToChannel(chMapX, data.ti.MapThr_x());
                    SendToChannel(chMapY, data.ti.MapRpm_y());
                    break;

                case MyECUData.types.AnalogInputs:
                    SendToChannel(chTPS, data.ai.Throttle);
                    ThrottleValue = (int) data.ai.Throttle;
                    SendToChannel(chOilT, data.ai.OilTemp);
                    SendToChannel(chAirT, data.ai.AirTemp);
                    SendToChannel(chBatV, data.ai.BatteryVolts);
                    SendToChannel(chAFR, MyECUData.lambda2AFR(MyECUData.voltage2Lambda(data.ai.WBO2Sensor0)));
                    SendToChannel(chLambda, MyECUData.voltage2Lambda(data.ai.WBO2Sensor0));
                    SendToChannel(chO2v, data.ai.WBO2Sensor0);
                    break;

                case MyECUData.types.CompensationInfo:
                    SendToChannel(chAirP, data.ci.AirPressure);
                    SendToChannel(chInjDur, data.ci.InjDuration * (data.ci.O2Adjust0)); // This one includes BaroAdj but not O2Adj so we add it
                    SendToChannel(chBaroAdj, data.ci.BaroAdjust * 100);
                    if (data.ci.ClosedLoopActive0 == 1)
                    {
                        SendToChannel(chTargAFR0, MyECUData.lambda2AFR(MyECUData.voltage2Lambda(data.ci.O2Target0)));
                        SendToChannel(chTargLambda0, MyECUData.voltage2Lambda(data.ci.O2Target0));
                        //SendToChannel(chTargV, data.ci.O2Target0);
                        SendToChannel(chO2Adjust0, data.ci.O2Adjust0 * 100);
                    }
                    else
                    {
                        SendToChannel(chTargAFR0, MyECUData.lambda2AFR(MyECUData.voltage2Lambda(data.ci.O2Target0)));
                        SendToChannel(chTargLambda0, MyECUData.voltage2Lambda(data.ci.O2Target0));
                        //SendToChannel(chTargV, data.ci.O2Target0);
                        SendToChannel(chO2Adjust0, 0);
                    }
                    DeadTime = data.ci.InjDeadTime; // We need to feed this around for calculations
                    break;

                case MyECUData.types.VersionInfo:
                    VersionString = data.vi.Version;
                    break;

                case MyECUData.types.DiagnosticInfo:
                    numalerts++;
                    break;

                default:
                    break;
            }
        }

        private void UpdateUIValues()
        {
            int t = com.Timeouts();
            if (t > 0)
            {
                textBox2.Text = "Alerts: " + numalerts.ToString() + " Timeouts: " + t.ToString();
            }
            else
            {
                textBox2.Text = "Alerts: " + numalerts.ToString();
            }
            progressBarTHR.Value = ThrottleValue;
            textBox1.Text = VersionString;
            if (dispatcherStopping)
            {
                dispatcherStopping = false;
                teardown();
                MessageBox.Show("End of Data");
            }
        }

        private void SendToChannel(int channel, double value)
        {
            if (channel == -1)
            {
                return;
            }
            else
            {
                axLW2VDev1.SetValue(channel, value);
            }
        }

        private void lwConnect()
        {
            if (!lwChannelsCreated)
            {
                if (axLW2VDev1.IsConnected == 0)
                {
                    /*
                     *   AddChannel is declared as:
                     *
                     *   STDMETHODIMP LW2VDev::addChannel(BSTR name, BSTR units, double min, double max, int type);
                     *    
                     *   name is the name of the channel
                     *   units is a string that represents the units
                     *   min and max are the minimum and maximum values for the channel (as floating point numbers)
                     *   type is 0=5VDC, 1=AFR/Lambda
                     *    
                     *   The Ex version of the call has the following additional params (after type):
                     *    
                     *   BSTR devname, int devinput, int devtype, int devfunc, int *chan.
                     *    
                     *   devname is a string representing the device name of the plug in, it should be 8 characters or less.
                     *   devinput is the physical input number on the device that the channel represents, it can be 0-255.
                     *   devtype is a 'sub type' for the device, it can be 0-255.
                     *   devfunc is a the current function setting for the channel, it can be 0-255.
                     *   chan is a integer for a returned value, it tells you what channel ID has been assigned for the created channel.
                     *    
                     *   The whole reason to use the first four is to have LogWorks cache user settings for the channel and recover them
                     *   in subsequent runs. Devname can be used to identify a specific plug in, and input, type, and function, can help
                     *   identify specifics. For example, at least one of the plug ins I wrote use them to store the ECU PID associated
                     *   with a channel and the flavor of protocol used. This way, if that configuration comes back, and user changes to
                     *   the channel in LogWorks are recovered.
                     */

                    string devname = "MyECU";   // this will show up in LogWorks
                    int devfunc = 0;            // we just increment it for each channel below
                    int devtype = 0xE609;       // magic number, for caching channel setup

                    // ti
                    axLW2VDev1.AddChannelEx("RPM", "rpm", 0, 10239, 0, devname, devtype, devfunc++, 0, ref chRPM);
                    axLW2VDev1.AddChannelEx("InjDur", "µs", 0, 16383, 0, devname, devtype, devfunc++, 0, ref chInjDur);
                    axLW2VDev1.AddChannelEx("SpkAdv", "°BTDC", 0, 90, 0, devname, devtype, devfunc++, 0, ref chSpkAdv);
                    axLW2VDev1.AddChannelEx("StartInj", "°BTDC", 0, 1023, 0, devname, devtype, devfunc++, 0, ref chSoI);
                    axLW2VDev1.AddChannelEx("EndOfInj", "°BTDC", 0, 1023, 0, devname, devtype, devfunc++, 0, ref chEoI);
                    axLW2VDev1.AddChannelEx("InjDutyC", "%", 0, 100, 0, devname, devtype, devfunc++, 0, ref chDutyCycle);
                    axLW2VDev1.AddChannelEx("MapRpmX", "index", 0, 15, 0, devname, devtype, devfunc++, 0, ref chMapX);
                    axLW2VDev1.AddChannelEx("MapThrY", "index", 0, 15, 0, devname, devtype, devfunc++, 0, ref chMapY);

                    // ai
                    axLW2VDev1.AddChannelEx("TPS", "TPS", 0, 1023, 0, devname, devtype, devfunc++, 0, ref chTPS);
                    axLW2VDev1.AddChannelEx("OilT", "°C", -30, 130, 0, devname, devtype, devfunc++, 0, ref chOilT);
                    axLW2VDev1.AddChannelEx("AirT", "°C", -30, 130, 0, devname, devtype, devfunc++, 0, ref chAirT);
                    axLW2VDev1.AddChannelEx("BatV", "V", 0, 16, 0, devname, devtype, devfunc++, 0, ref chBatV);
                    axLW2VDev1.AddChannelEx("AFR", "AFR", 7.35, 22.39, 0, devname, devtype, devfunc++, 0, ref chAFR);
                    axLW2VDev1.AddChannelEx("Lambda", "Lambda", 0.5, 1.523, 0, devname, devtype, devfunc++, 0, ref chLambda);
                    axLW2VDev1.AddChannelEx("O2sensor", "V", 0, 5, 0, devname, devtype, devfunc++, 0, ref chO2v);
                    
                    // ci
                    axLW2VDev1.AddChannelEx("AirP", "mBar", 256, 1279, 0, devname, devtype, devfunc++, 0, ref chAirP);
                    axLW2VDev1.AddChannelEx("TargAFR", "AFR", 7.35, 22.39, 0, devname, devtype, devfunc++, 0, ref chTargAFR0);
                    axLW2VDev1.AddChannelEx("TargLmbd", "Lambda", 0.5, 1.523, 0, devname, devtype, devfunc++, 0, ref chTargLambda0);
                    //axLW2VDev1.AddChannelEx("TargO2", "V", 0, 5, 0, devname, devtype, devfunc++, 0, ref chTargV);
                    axLW2VDev1.AddChannelEx("CLadj", "%", 50, 150, 0, devname, devtype, devfunc++, 0, ref chO2Adjust0);
                    axLW2VDev1.AddChannelEx("BaroAdj", "%", 50, 150, 0, devname, devtype, devfunc++, 0, ref chBaroAdj);
                }
                else
                {
                    MessageBox.Show("Close Plugin and LogWorks and restart.\nConnect Plugin before starting LogWorks.");
                    buttonConnect.Enabled = false;
                    buttonConnect.Text = "error";
                    return;
                }
            }

            if (chRPM == -1)
            {
                MessageBox.Show("Failed adding channels to LogWorks\nClose Plugin and LogWorks and restart.\nConnect Plugin before starting LogWorks.");
                buttonConnect.Enabled = false;
                buttonConnect.Text = "error";
                return;
            }
            else
            {
                lwChannelsCreated = true;
            }
        }

        private void buttonAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("MyECU plugin for Innovate LogWorks 3\n" +
                "(threaded version)\n\n" +
                "R. Andersson, A. Hedin 2009\n" +
                "\nFeel free to re-distribute, this is FREE software and you only get what you paid for.\n" +
                "\nhttp://www.abc.se/~rasmus/logworks/"
                );
        }

        private void TimerUpdatePortList_Tick(object sender, EventArgs e)
        {
            UpdatePortList();
        }

        private void TimerUIupdate_Tick(object sender, EventArgs e)
        {
            UpdateUIValues();
        }

    }
}
